-- Sistemas Integrados TEKNISA
-- Faturamento de Servios
--
-- Nome: 0000_FUNTEKO_FPA_PONTO.sql
--
-- Descrio:
-- Retorna as informaes de ponto do dia de um funcionario consolidadas e totalizadas
--
-- Parmetros:
-- P_CDEMPRESA - Cdigo da empresa
-- P_CDFUNC - Cdigo do funcionrio
-- P_IDORIGEM - Indica se a funo dever buscar os dados da tabela BATIDAPTO ou FPA_AUSENCIAPONTO
--
-- Exemplo de utilizao
-- SELECT * 
--   FROM TABLE (FPA_PONTO_DIA (:P_CDEMPRESA, :P_CDFUNC, :P_DTBATIDAPTO, :P_IDORIGEM))
--
-- Criao:
-- 19/09/2012 Newton Neves dos Santos

DROP FUNCTION FPA_PONTO_DIA
/
DROP TYPE FPA_PONTO_DIA_TABLE
/
DROP TYPE FPA_PONTO_DIA_TYPE
/

CREATE TYPE FPA_PONTO_DIA_TYPE AS OBJECT 
   (NRPONTO NUMBER,
    CDEMPRESA VARCHAR2(02),
    CDFUNC VARCHAR2(10),
    IDOCORRENCIA VARCHAR2(20),  
    DTHRENTRADAORIG DATE,
    DTHRENTRADA DATE,
    HRENTRADA VARCHAR2(4),
    DTHRSAIDAORIG DATE,
    DTHRSAIDA DATE,
    HRSAIDA VARCHAR2(4),
    HRAUSENCIAABONAORIG VARCHAR2(4),
    HRAUSENCIAEXTABONORIG VARCHAR2(4),
    HRAUSENCIANAOABONORIG  VARCHAR2(4),
    HRTEMPOCALCULADO VARCHAR2(5),
    HRTEMPOCALCORIG VARCHAR2(5),
    IDALMOCO VARCHAR2(01),
    IDTEMPOABONADO VARCHAR2(01),
    IDTEMPONAOABON VARCHAR2(01),
    DSOBSERVACAO VARCHAR2(250),
    DSALTEENTRADA VARCHAR2(250),
    DSALTESAIDA VARCHAR2(250),
    DSALTEAUSENCIAABONADA VARCHAR2(250),
    DSALTEAUSENCIAEXTABON VARCHAR2(250),
    DSALTEAUSENCIANAOABON VARCHAR2(250))
/

CREATE TYPE FPA_PONTO_DIA_TABLE AS TABLE OF FPA_PONTO_DIA_TYPE
/

CREATE OR REPLACE 
FUNCTION FPA_PONTO_DIA
  (P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
   P_CDFUNC IN FUNCIONARIO.CDFUNC%TYPE,
   P_DTBATIDAPTO IN BATIDAPONTO.DTBATIDAPTO%TYPE,
   P_IDORIGEM IN VARCHAR2,
   P_DTBATIDAINI IN BATIDAPONTO.DTBATIDAPTO%TYPE DEFAULT NULL,
   P_DTBATIDAFIM IN BATIDAPONTO.DTBATIDAPTO%TYPE DEFAULT NULL) 
RETURN FPA_PONTO_DIA_TABLE PIPELINED 

IS
   OUT_RECORD FPA_PONTO_DIA_TYPE := FPA_PONTO_DIA_TYPE (NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
   
   V_DTAUX DATE;
   V_UTILIZABATPONTO BOOLEAN;
   V_CDTPADMISSAO FUNCIONARIO.CDTPADMISSAO%TYPE;
   V_SGESTADO_FUNC ENDEFILI.SGESTADO%TYPE;
   V_IDSENTIDO BATIDAPONTO.IDSENTBATIDA%TYPE;
   V_IDULTSENTIDO BATIDAPONTO.IDSENTBATIDA%TYPE;
   V_AUSENCIAALMOCO VARCHAR2(04);
   V_AUSENCIATOTAL VARCHAR2(04);
   V_AUSENCIAABONADA VARCHAR2(04);
   V_AUSENCIAEXTABON VARCHAR2(04);
   V_AUSENCIANAOABON VARCHAR2(04);
   V_HORASREMUNERADAS VARCHAR2(04);
   V_QTHORASBASE VARCHAR2(04);
   V_ALMOCOHORABASE VARCHAR2(04);
   V_DEBITOCREDITOHORAS VARCHAR2(05);
   V_DEBITOCREDITOHORASEXTRA VARCHAR2(05);
   
   V_DTBATIDAINI BATIDAPONTO.DTBATIDAPTO%TYPE;
   V_DTBATIDAFIM BATIDAPONTO.DTBATIDAPTO%TYPE;
   
   V_DTHBENTRADA DATE;
   V_DTHBSAIDA DATE;
   V_DTHBENTRADATOLERA DATE;
   V_HBENTRADA VARCHAR2(4);
   V_HBSAIDAALMOCO VARCHAR2(4);
   V_HBVOLTAALMOCO VARCHAR2(4);
   V_HBSAIDA VARCHAR2(4);
   
   V_NRPONTO FPA_PONTO.NRPONTO%TYPE;
   V_DTHRENTRADAORIG BATIDAPONTO.DTBATIDAPTO%TYPE;
   V_DTHRENTRADA BATIDAPONTO.DTBATIDAPTO%TYPE;
   V_HRENTRADA VARCHAR2(4);
   V_DTHRSAIDAORIG BATIDAPONTO.DTBATIDAPTO%TYPE;
   V_DTHRSAIDA BATIDAPONTO.DTBATIDAPTO%TYPE;
   V_HRSAIDA VARCHAR2(4);
   V_HRAUSENCIAABONADAORIG VARCHAR2(4);
   V_HRAUSENCIAEXTABONORIG VARCHAR2(4);
   V_HRAUSENCIANAOABONORIG VARCHAR2(4);
   V_HRTEMPOCALCORIG VARCHAR2(4);
   V_HRTEMPOCALCULADO VARCHAR2(4);
   V_DEBCREDHORASORIG VARCHAR2(05);
   V_DEBCREDHREXTRAORIG VARCHAR2(05);
   V_IDALMOCO VARCHAR2(01);
   V_IDTEMPOABONADO VARCHAR2(01);
   V_IDTEMPONAOABON VARCHAR2(01);
   V_DSOBSERVACAO VARCHAR2(250);
   V_DSALTEENTRADA VARCHAR2(250);
   V_DSALTESAIDA VARCHAR2(250);
   V_DSALTEAUSENCIAABONADA VARCHAR2(250);
   V_DSALTEAUSENCIANAOABON VARCHAR2(250);
   V_DSALTEAUSENCIAEXTABON VARCHAR2(250);
      
   V_IDSENTBATIDA BATIDAPONTO.IDSENTBATIDA%TYPE;
   V_DTBATIDAPTO BATIDAPONTO.DTBATIDAPTO%TYPE;
   V_SAB_DOM_FERIADO VARCHAR2(01);
   
   CURSOR C_BATPONTO IS 
       SELECT IDSENTBATIDA, DTBATIDAPTO 
         FROM BATIDAPONTO  
        WHERE CDFUNC = P_CDFUNC
          AND CDEMPRESA = P_CDEMPRESA
          AND DTBATIDAPTO > V_DTBATIDAINI 
          AND DTBATIDAPTO < V_DTBATIDAFIM
        ORDER BY DTBATIDAPTO;

   CURSOR C_FPAPONTO IS 
       SELECT NRAUSENCIAPONTO, DTINICIOAUSENCIA, DTFIMAUSENCIA, HRAUSENCIA, 
              IDAUSENCIANAOABON, IDAUSENCIAABONADA, IDAUSENCIAALMOCO, DSOBSERVACAOAUSENCIA
         FROM FPA_AUSENCIAPONTO
        WHERE CDEMPRESA = P_CDEMPRESA
          AND CDFUNC = P_CDFUNC
          AND DTPONTO = P_DTBATIDAPTO
        ORDER BY DTINICIOAUSENCIA;

BEGIN
   /* ######################################### */
   /* INICIO - Verificao de dia til/no til */
   /* ------ */
   
   BEGIN
      SELECT DECODE(FF.DTFERIFILI, NULL, 'N', 'S')
        INTO V_SAB_DOM_FERIADO
        FROM FERIFILI FF, FUNCIONARIO FU
       WHERE FU.CDEMPRESA = P_CDEMPRESA 
         AND FU.CDFUNC = P_CDFUNC
         AND FF.CDFILIAL = FU.CDFILIALOCFUNC
         AND FF.DTFERIFILI = P_DTBATIDAPTO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_SAB_DOM_FERIADO := 'N';
      END;
   END;
      
   /* --- */
   /* FIM - Verificao de dia til/no til */
   /* ###################################### */

   /* ####################################################################### */
   /* INICIO - Verificao de informaes sobre o horrio base do funcionrio */
   /* ------ */
   
   BEGIN
      SELECT FUN.CDTPADMISSAO, EFI.SGESTADO,
             TO_DATE(TO_CHAR(P_DTBATIDAPTO, 'DD/MM/YYYY')||' '||TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRENT1HBFUN, HBE.HRENT1HBEXC), 'HH24:MI'), 'DD/MM/YYYY HH24:MI'), 
             TO_DATE(TO_CHAR(P_DTBATIDAPTO, 'DD/MM/YYYY')||' '||UTILIT.SUBT_HORA2(TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRENT1HBFUN, HBE.HRENT1HBEXC), 'HH24MI'), '0005'), 'DD/MM/YYYY HH24MI'), /* <--- Considera 5 minutos de tolerncia na 1a. entrada */
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRENT1HBFUN, HBE.HRENT1HBEXC), 'HH24MI'), 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRSAI1HBFUN, HBE.HRSAI1HBEXC), 'HH24MI'), 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRENT2HBFUN, HBE.HRENT2HBEXC), 'HH24MI'), 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRSAI2HBFUN, HBE.HRSAI2HBEXC), 'HH24MI'),
             TO_DATE(TO_CHAR(P_DTBATIDAPTO, 'DD/MM/YYYY')||' '||TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRSAI2HBFUN, HBE.HRSAI2HBEXC), 'HH24:MI'), 'DD/MM/YYYY HH24:MI') 
        INTO V_CDTPADMISSAO, V_SGESTADO_FUNC, V_DTHBENTRADA, V_DTHBENTRADATOLERA, V_HBENTRADA, V_HBSAIDAALMOCO, V_HBVOLTAALMOCO, V_HBSAIDA, V_DTHBSAIDA
        FROM FUNCIONARIO FUN, FPA_HORABASEEXC HBE, ENDEFILI EFI,
             (SELECT CDEMPRESA, CDFUNC, HRENT1HBFUN, HRSAI1HBFUN, HRENT2HBFUN, HRSAI2HBFUN
                FROM HORBASFU
               WHERE CDFUNC = P_CDFUNC
                 AND CDEMPRESA = P_CDEMPRESA
                 AND IDDIHBFUN = TO_CHAR(P_DTBATIDAPTO, 'D')
                 AND DTINICHBFUN = (SELECT MAX(DTINICHBFUN)
                                      FROM HORBASFU
                                     WHERE CDFUNC = P_CDFUNC
                                       AND CDEMPRESA = P_CDEMPRESA
                                       AND DTINICHBFUN <= P_DTBATIDAPTO)) HBF
       WHERE FUN.CDFUNC = P_CDFUNC
         AND FUN.CDEMPRESA = P_CDEMPRESA
         AND EFI.CDFILIAL(+) = FUN.CDFILIALOCFUNC
         AND EFI.IDTPENDEFILI = 'P'
         AND HBF.CDFUNC(+) = FUN.CDFUNC
         AND HBF.CDEMPRESA(+) = FUN.CDEMPRESA
         AND HBE.CDTPADMISSAO(+) = FUN.CDTPADMISSAO
         AND HBE.DTHORABASEEXC(+) = P_DTBATIDAPTO;

      IF (V_HBENTRADA IS NULL) OR (V_SAB_DOM_FERIADO = 'S') THEN
         V_QTHORASBASE := '0000';
         V_ALMOCOHORABASE := '0000';
      ELSE
         IF V_HBVOLTAALMOCO IS NULL THEN
            V_QTHORASBASE := UTILIT.SUBT_HORA2(V_HBSAIDAALMOCO, V_HBENTRADA);
            V_ALMOCOHORABASE := '0000';
         ELSE
            V_QTHORASBASE := UTILIT.SOMA_HORA(UTILIT.SUBT_HORA2(V_HBSAIDAALMOCO, V_HBENTRADA), UTILIT.SUBT_HORA2(V_HBSAIDA, V_HBVOLTAALMOCO));
            V_ALMOCOHORABASE := UTILIT.SUBT_HORA2(V_HBVOLTAALMOCO, V_HBSAIDAALMOCO);
         END IF;   
      END IF;      
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_QTHORASBASE := '0000';
         V_ALMOCOHORABASE := '0000';
      END;
   END;
      
   OUT_RECORD.NRPONTO := NULL;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'ALMOCOHORABASE';
   OUT_RECORD.DTHRENTRADAORIG := P_DTBATIDAPTO;
   OUT_RECORD.DTHRENTRADA := NULL;
   OUT_RECORD.HRENTRADA := V_HBENTRADA;
   OUT_RECORD.DTHRSAIDAORIG := NULL;
   OUT_RECORD.DTHRSAIDA := NULL;
   OUT_RECORD.HRSAIDA := NVL(V_HBSAIDA, V_HBSAIDAALMOCO);
   OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
   OUT_RECORD.HRTEMPOCALCULADO := V_ALMOCOHORABASE;
   OUT_RECORD.HRTEMPOCALCORIG := NULL;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   PIPE ROW (OUT_RECORD);

   /* --- */
   /* FIM - Verificao de informaes sobre o horrio base do funcionrio */
   /* #################################################################### */

   V_UTILIZABATPONTO := FALSE;
   
   IF P_IDORIGEM = 'FPA_PONTO' THEN
      BEGIN
         SELECT NRPONTO, DTENTRADAORIG, DTENTRADA, DTSAIDAORIG, DTSAIDA, HRAUSENCIAALMOCO, HRAUSENCIATOTAL, HRAUSENCIANAOABONORIG, HRAUSENCIANAOABON, HRAUSENCIAEXTABONORIG, HRAUSENCIAEXTABON,
                HRAUSENCIAABONADAORIG, HRAUSENCIAABONADA, HRREMUNERADA, HRSALDO, HRSALDOHORAEXTRA, 
                DSALTEENTRADA, DSALTESAIDA, DSALTEAUSENCIANAOABON, DSALTEAUSENCIAABONADA, DSALTEAUSENCIAEXTABON
           INTO V_NRPONTO, V_DTHRENTRADAORIG, V_DTHRENTRADA, V_DTHRSAIDAORIG, V_DTHRSAIDA, V_AUSENCIAALMOCO, V_AUSENCIATOTAL, V_HRAUSENCIANAOABONORIG, V_AUSENCIANAOABON, V_HRAUSENCIAEXTABONORIG, V_AUSENCIAEXTABON,
                V_HRAUSENCIAABONADAORIG, V_AUSENCIAABONADA, V_HORASREMUNERADAS, V_DEBITOCREDITOHORAS, V_DEBITOCREDITOHORASEXTRA, 
                V_DSALTEENTRADA, V_DSALTESAIDA, V_DSALTEAUSENCIANAOABON, V_DSALTEAUSENCIAABONADA, V_DSALTEAUSENCIAEXTABON
           FROM FPA_PONTO
          WHERE CDEMPRESA = P_CDEMPRESA
            AND CDFUNC = P_CDFUNC
            AND DTPONTO = P_DTBATIDAPTO;
            
         V_HRENTRADA := TO_CHAR(V_DTHRENTRADA, 'HH24MI');
         V_HRSAIDA := TO_CHAR(V_DTHRSAIDA, 'HH24MI');
         V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRSAIDA, V_HRENTRADA);
         
         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'ENTRADASAIDA';
         OUT_RECORD.DTHRENTRADAORIG := V_DTHRENTRADAORIG;
         OUT_RECORD.DTHRENTRADA := V_DTHRENTRADA;
         OUT_RECORD.HRENTRADA := V_HRENTRADA;
         OUT_RECORD.DTHRSAIDAORIG := V_DTHRSAIDAORIG;
         OUT_RECORD.DTHRSAIDA := V_DTHRSAIDA;
         OUT_RECORD.HRSAIDA := V_HRSAIDA;
         OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
         OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
         OUT_RECORD.HRTEMPOCALCULADO := V_HRTEMPOCALCULADO;
         OUT_RECORD.HRTEMPOCALCORIG := NULL;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := V_DSALTEENTRADA;
         OUT_RECORD.DSALTESAIDA := V_DSALTESAIDA;
         OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
         OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
         PIPE ROW (OUT_RECORD);
         
         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'AUSENCIAALMOCO';
         OUT_RECORD.DTHRENTRADAORIG := NULL;
         OUT_RECORD.DTHRENTRADA := NULL;
         OUT_RECORD.HRENTRADA := NULL;
         OUT_RECORD.DTHRSAIDAORIG := NULL;
         OUT_RECORD.DTHRSAIDA := NULL;
         OUT_RECORD.HRSAIDA := NULL;
         OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
         OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
         OUT_RECORD.HRTEMPOCALCULADO := V_AUSENCIAALMOCO;
         OUT_RECORD.HRTEMPOCALCORIG := NULL;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := NULL;
         OUT_RECORD.DSALTESAIDA := NULL;
         OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
         OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
         PIPE ROW (OUT_RECORD);
         
         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'AUSENCIAABONADA';
         OUT_RECORD.DTHRENTRADAORIG := NULL;
         OUT_RECORD.DTHRENTRADA := NULL;
         OUT_RECORD.HRENTRADA := NULL;
         OUT_RECORD.DTHRSAIDAORIG := NULL;
         OUT_RECORD.DTHRSAIDA := NULL;
         OUT_RECORD.HRSAIDA := NULL;
         OUT_RECORD.HRAUSENCIAABONAORIG := V_HRAUSENCIAABONADAORIG;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
         OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
         OUT_RECORD.HRTEMPOCALCULADO := V_AUSENCIAABONADA;
         OUT_RECORD.HRTEMPOCALCORIG := NULL;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := NULL;
         OUT_RECORD.DSALTESAIDA := NULL;
         OUT_RECORD.DSALTEAUSENCIAABONADA := V_DSALTEAUSENCIAABONADA;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
         OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
         PIPE ROW (OUT_RECORD);
         
         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'AUSENCIAEXTABONADA';
         OUT_RECORD.DTHRENTRADAORIG := NULL;
         OUT_RECORD.DTHRENTRADA := NULL;
         OUT_RECORD.HRENTRADA := NULL;
         OUT_RECORD.DTHRSAIDAORIG := NULL;
         OUT_RECORD.DTHRSAIDA := NULL;
         OUT_RECORD.HRSAIDA := NULL;
         OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := V_HRAUSENCIAEXTABONORIG;
         OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
         OUT_RECORD.HRTEMPOCALCULADO := V_AUSENCIAEXTABON;
         OUT_RECORD.HRTEMPOCALCORIG := NULL;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := NULL;
         OUT_RECORD.DSALTESAIDA := NULL;
         OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := V_DSALTEAUSENCIAEXTABON;
         OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
         PIPE ROW (OUT_RECORD);
         
         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'AUSENCIANAOABONADA';
         OUT_RECORD.DTHRENTRADAORIG := NULL;
         OUT_RECORD.DTHRENTRADA := NULL;
         OUT_RECORD.HRENTRADA := NULL;
         OUT_RECORD.DTHRSAIDAORIG := NULL;
         OUT_RECORD.DTHRSAIDA := NULL;
         OUT_RECORD.HRSAIDA := NULL;
         OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
         OUT_RECORD.HRAUSENCIANAOABONORIG := V_HRAUSENCIANAOABONORIG;
         OUT_RECORD.HRTEMPOCALCULADO := V_AUSENCIANAOABON;
         OUT_RECORD.HRTEMPOCALCORIG := NULL;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := NULL;
         OUT_RECORD.DSALTESAIDA := NULL;
         OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
         OUT_RECORD.DSALTEAUSENCIANAOABON := V_DSALTEAUSENCIANAOABON;
         PIPE ROW (OUT_RECORD);
         
         /* Clculo do tempo total de ausncia baseado nos dados originais */
         V_HRTEMPOCALCORIG := UTILIT.SOMA_HORA(V_HRAUSENCIANAOABONORIG, V_HRAUSENCIAABONADAORIG);

         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'AUSENCIATOTAL';
         OUT_RECORD.DTHRENTRADAORIG := NULL;
         OUT_RECORD.DTHRENTRADA := NULL;
         OUT_RECORD.HRENTRADA := NULL;
         OUT_RECORD.DTHRSAIDAORIG := NULL;
         OUT_RECORD.DTHRSAIDA := NULL;
         OUT_RECORD.HRSAIDA := NULL;
         OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
         OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
         OUT_RECORD.HRTEMPOCALCULADO := V_AUSENCIATOTAL;
         OUT_RECORD.HRTEMPOCALCORIG := V_HRTEMPOCALCORIG;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := NULL;
         OUT_RECORD.DSALTESAIDA := NULL;
         OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
         OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
         PIPE ROW (OUT_RECORD);
         
         /* Clculo das horas remunradas baseadas nos dados originais */
         V_HRTEMPOCALCORIG := UTILIT.SUBT_HORA2(TO_CHAR(V_DTHRSAIDAORIG, 'HH24MI'), TO_CHAR(V_DTHRENTRADAORIG, 'HH24MI'));
         V_HRTEMPOCALCORIG := UTILIT.SUBT_HORA2(V_HRTEMPOCALCORIG, V_AUSENCIAALMOCO);
         V_HRTEMPOCALCORIG := UTILIT.SUBT_HORA2(V_HRTEMPOCALCORIG, V_HRAUSENCIANAOABONORIG);
         V_HRTEMPOCALCORIG := UTILIT.SOMA_HORA(V_HRTEMPOCALCORIG, V_HRAUSENCIAEXTABONORIG);

         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'HORASREMUNERADAS';
         OUT_RECORD.DTHRENTRADAORIG := NULL;
         OUT_RECORD.DTHRENTRADA := NULL;
         OUT_RECORD.HRENTRADA := NULL;
         OUT_RECORD.DTHRSAIDAORIG := NULL;
         OUT_RECORD.DTHRSAIDA := NULL;
         OUT_RECORD.HRSAIDA := NULL;
         OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
         OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
         OUT_RECORD.HRTEMPOCALCULADO := V_HORASREMUNERADAS;
         OUT_RECORD.HRTEMPOCALCORIG := V_HRTEMPOCALCORIG;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := NULL;
         OUT_RECORD.DSALTESAIDA := NULL;
         OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
         OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
         PIPE ROW (OUT_RECORD);

         /* Clculo do dbito/crdito de horas (banco de horas e horas extra) baseadas nos dados originais */
         IF V_HRTEMPOCALCORIG > V_QTHORASBASE THEN
            IF (V_CDTPADMISSAO IN ('S', 'P')) THEN
               V_DEBCREDHORASORIG := '0000';
            ELSE
               V_DEBCREDHORASORIG := UTILIT.SUBT_HORA2(V_HRTEMPOCALCORIG, V_QTHORASBASE);
            END IF;   
             
            /* #### Clculo atual considerando banco de horas #### */
            V_DEBCREDHREXTRAORIG := ' 0000';
            
            /* #### Clculo considerando horas extras a partir de 02:00 horas positivas (funcionrios alocados em MG) 
				 * ou todo saldo positivo como hora extra (funcionrios alocados em outros estados) 
				 * ou todo saldo positivo como hora extra (Sb, Dom e Feriados - Solicitado por Gabrielle em 13/11/2014) #### */
            IF NOT (V_CDTPADMISSAO IN ('B', 'E', 'O')) THEN                
               IF V_SGESTADO_FUNC = 'MG' THEN
                  IF (NOT (V_CDTPADMISSAO IN ('S', 'P'))) THEN
                     IF (V_SAB_DOM_FERIADO = 'N') THEN 
								IF (V_DEBCREDHORASORIG > '0200') THEN
									V_DEBCREDHREXTRAORIG := UTILIT.SUBT_HORA2(V_DEBCREDHORASORIG, '0200');
								END IF;   
							ELSE
								V_DEBCREDHREXTRAORIG := V_DEBCREDHORASORIG;
							END IF;	
                  END IF;   
               ELSE
                  V_DEBCREDHREXTRAORIG := V_DEBCREDHORASORIG;
               END IF;               
            END IF;   
         ELSE
            V_DEBCREDHREXTRAORIG := ' 0000';
            IF V_HRTEMPOCALCORIG < V_QTHORASBASE THEN
               V_DEBCREDHORASORIG := '-'||UTILIT.SUBT_HORA2(V_QTHORASBASE, V_HRTEMPOCALCORIG);
            ELSE
               V_DEBCREDHORASORIG := ' '||UTILIT.SUBT_HORA2(V_QTHORASBASE, V_HRTEMPOCALCORIG);
            END IF;   
         END IF;   

         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'DEBITOCREDITOHORAS';
         OUT_RECORD.DTHRENTRADAORIG := NULL;
         OUT_RECORD.DTHRENTRADA := NULL;
         OUT_RECORD.HRENTRADA := NULL;
         OUT_RECORD.DTHRSAIDAORIG := NULL;
         OUT_RECORD.DTHRSAIDA := NULL;
         OUT_RECORD.HRSAIDA := NULL;
         OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
         OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
         OUT_RECORD.HRTEMPOCALCULADO := V_DEBITOCREDITOHORAS;
         OUT_RECORD.HRTEMPOCALCORIG := V_DEBCREDHORASORIG;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := NULL;
         OUT_RECORD.DSALTESAIDA := NULL;
         OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
         OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
         PIPE ROW (OUT_RECORD);

         OUT_RECORD.NRPONTO := V_NRPONTO;
         OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
         OUT_RECORD.CDFUNC := P_CDFUNC;
         OUT_RECORD.IDOCORRENCIA := 'DEBCREDHORASEXTRA';
         OUT_RECORD.DTHRENTRADAORIG := NULL;
         OUT_RECORD.DTHRENTRADA := NULL;
         OUT_RECORD.HRENTRADA := NULL;
         OUT_RECORD.DTHRSAIDAORIG := NULL;
         OUT_RECORD.DTHRSAIDA := NULL;
         OUT_RECORD.HRSAIDA := NULL;
         OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
         OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
         OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
         OUT_RECORD.HRTEMPOCALCULADO := V_DEBITOCREDITOHORASEXTRA;
         OUT_RECORD.HRTEMPOCALCORIG := V_DEBCREDHREXTRAORIG;
         OUT_RECORD.IDALMOCO := 'N';
         OUT_RECORD.IDTEMPOABONADO := 'N';
         OUT_RECORD.IDTEMPONAOABON := 'N';
         OUT_RECORD.DSOBSERVACAO := NULL;
         OUT_RECORD.DSALTEENTRADA := NULL;
         OUT_RECORD.DSALTESAIDA := NULL;
         OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
         OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
         OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
         PIPE ROW (OUT_RECORD);

         OPEN C_FPAPONTO;
         LOOP
            FETCH C_FPAPONTO INTO V_NRPONTO, V_DTHRSAIDA, V_DTHRENTRADA, V_HRTEMPOCALCULADO, V_IDTEMPONAOABON, V_IDTEMPOABONADO, V_IDALMOCO, V_DSOBSERVACAO;
            EXIT WHEN C_FPAPONTO%NOTFOUND;   

               V_HRENTRADA := TO_CHAR(V_DTHRENTRADA, 'HH24MI');
               V_HRSAIDA := TO_CHAR(V_DTHRSAIDA, 'HH24MI');
            
               OUT_RECORD.NRPONTO := V_NRPONTO;
               OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
               OUT_RECORD.CDFUNC := P_CDFUNC;
               OUT_RECORD.IDOCORRENCIA := 'AUSENCIA';
               OUT_RECORD.DTHRENTRADAORIG := NULL;
               OUT_RECORD.DTHRENTRADA := V_DTHRENTRADA;
               OUT_RECORD.HRENTRADA := V_HRENTRADA;
               OUT_RECORD.DTHRSAIDAORIG := NULL;
               OUT_RECORD.DTHRSAIDA := V_DTHRSAIDA;
               OUT_RECORD.HRSAIDA := V_HRSAIDA;
               OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
               OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
               OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
               OUT_RECORD.HRTEMPOCALCULADO := V_HRTEMPOCALCULADO;
               OUT_RECORD.HRTEMPOCALCORIG := NULL;
               OUT_RECORD.IDALMOCO := V_IDALMOCO;
               OUT_RECORD.IDTEMPOABONADO := V_IDTEMPOABONADO;
               OUT_RECORD.IDTEMPONAOABON := V_IDTEMPONAOABON;
               OUT_RECORD.DSOBSERVACAO := V_DSOBSERVACAO;
               OUT_RECORD.DSALTEENTRADA := NULL;
               OUT_RECORD.DSALTESAIDA := NULL;
               OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
               OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
               OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
               PIPE ROW (OUT_RECORD);
               
         END LOOP;
         CLOSE C_FPAPONTO;
         
      EXCEPTION
         WHEN NO_DATA_FOUND THEN BEGIN
            V_UTILIZABATPONTO := TRUE;
         END;
      END;
   ELSE
      V_UTILIZABATPONTO := TRUE;
   END IF;   
      
   IF NOT V_UTILIZABATPONTO THEN
      RETURN;
   END IF;

   /* ################################################################## */
   /* INICIO - Definio de perodo a ser considerado quando BATIDAPONTO */
   /* ------ */
   
   IF P_DTBATIDAINI IS NULL THEN
      V_DTBATIDAINI := TO_DATE(TO_CHAR(P_DTBATIDAPTO, 'DD/MM/YYYY')||' 04:00:01', 'DD/MM/YYYY HH24:MI:SS');
   ELSE
      V_DTBATIDAINI := P_DTBATIDAINI;
   END IF;

   IF P_DTBATIDAFIM IS NULL THEN
      V_DTBATIDAFIM := TO_DATE(TO_CHAR(P_DTBATIDAPTO+1, 'DD/MM/YYYY')||' 04:00:00', 'DD/MM/YYYY HH24:MI:SS');
   ELSE
      V_DTBATIDAFIM := P_DTBATIDAFIM;
   END IF;
   
   BEGIN
      SELECT IDSENTBATIDA, DTBATIDAPTO
        INTO V_IDSENTIDO, V_DTAUX
        FROM BATIDAPONTO  
       WHERE CDFUNC = P_CDFUNC
         AND CDEMPRESA = P_CDEMPRESA
         AND DTBATIDAPTO = (SELECT MIN(DTBATIDAPTO)
                              FROM BATIDAPONTO
                             WHERE CDEMPRESA = P_CDEMPRESA
                               AND CDFUNC = P_CDFUNC
                               AND DTBATIDAPTO >= V_DTBATIDAFIM);
                               
      IF V_IDSENTIDO = '2' THEN
         V_DTBATIDAFIM := V_DTAUX;
      END IF;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_DTBATIDAFIM := V_DTBATIDAFIM;
      END;
   END;
   
   /* --- */
   /* FIM - Definio de perodo a ser considerado quando BATIDAPONTO */
   /* ############################################################### */

   SELECT M.DTENTRADA, CASE WHEN E.DTENTRADA < V_DTHBENTRADATOLERA THEN V_DTHBENTRADA ELSE E.DTENTRADA END, 
          S.DTSAIDA
     INTO V_DTAUX, V_DTHRENTRADA, V_DTHRSAIDA
     FROM (SELECT MAX(DTBATIDAPTO) AS DTENTRADA
             FROM BATIDAPONTO  
            WHERE CDEMPRESA = P_CDEMPRESA
              AND CDFUNC = P_CDFUNC
              AND IDSENTBATIDA = '1'
              AND DTBATIDAPTO BETWEEN V_DTBATIDAINI AND V_DTBATIDAFIM) M,
          (SELECT MIN(DTBATIDAPTO) AS DTENTRADA
             FROM BATIDAPONTO  
            WHERE CDEMPRESA = P_CDEMPRESA
              AND CDFUNC = P_CDFUNC
              AND IDSENTBATIDA = '1'
              AND DTBATIDAPTO BETWEEN V_DTBATIDAINI AND V_DTBATIDAFIM) E,
          (SELECT MAX(DTBATIDAPTO) AS DTSAIDA
             FROM BATIDAPONTO  
            WHERE CDEMPRESA = P_CDEMPRESA
              AND CDFUNC = P_CDFUNC
              AND IDSENTBATIDA = '2'
              AND DTBATIDAPTO BETWEEN V_DTBATIDAINI AND V_DTBATIDAFIM) S;
                 
   IF V_DTHRENTRADA IS NULL OR V_DTHRSAIDA IS NULL THEN
      IF (NOT V_DTHRENTRADA IS NULL) AND (V_DTHRSAIDA IS NULL) AND (P_DTBATIDAPTO = TRUNC(SYSDATE)) THEN
         V_DTHRSAIDA :=  V_DTBATIDAFIM;
      ELSE
         RETURN; 
      END IF;
   ELSE
      IF V_DTAUX > V_DTHRSAIDA THEN
         V_DTHRSAIDA :=  V_DTBATIDAFIM;
      END IF;       
   END IF;   
   
   V_HRENTRADA := TO_CHAR(V_DTHRENTRADA, 'HH24MI');
   V_HRSAIDA := TO_CHAR(V_DTHRSAIDA, 'HH24MI');
   V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRSAIDA, V_HRENTRADA);
              
   OUT_RECORD.NRPONTO := NULL;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'ENTRADASAIDA';
   OUT_RECORD.DTHRENTRADAORIG := V_DTHRENTRADA;
   OUT_RECORD.DTHRENTRADA := V_DTHRENTRADA;
   OUT_RECORD.HRENTRADA := V_HRENTRADA;
   OUT_RECORD.DTHRSAIDAORIG := V_DTHRSAIDA;
   OUT_RECORD.DTHRSAIDA := V_DTHRSAIDA;
   OUT_RECORD.HRSAIDA := V_HRSAIDA;
   OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
   OUT_RECORD.HRTEMPOCALCULADO := V_HRTEMPOCALCULADO;
   OUT_RECORD.HRTEMPOCALCORIG := NULL;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   PIPE ROW (OUT_RECORD);
        
   V_HORASREMUNERADAS := V_HRTEMPOCALCULADO;
   
   V_IDULTSENTIDO := '1';

   /* Inicializao de variaveis para loop no cursor C_BATPONTO */
   V_DTBATIDAINI := V_DTHRENTRADA;
   V_DTBATIDAFIM := V_DTHRSAIDA;
   V_AUSENCIATOTAL := '0000';
   
   /* Variveis de sada inicializadas com valores de entrada 
      para o caso de entrada anterior a hora da 1a. entrada base. */
   V_DTHRSAIDA := V_DTHRENTRADA;
   V_HRSAIDA := TO_CHAR(V_DTHRENTRADA, 'HH24MI');
   
   OPEN C_BATPONTO;
   LOOP
      FETCH C_BATPONTO INTO V_IDSENTBATIDA, V_DTBATIDAPTO;
      EXIT WHEN C_BATPONTO%NOTFOUND;   
         
         IF (V_IDSENTBATIDA = '1') AND (V_IDULTSENTIDO <> '1') THEN
            V_DTHRENTRADA := V_DTBATIDAPTO;
            V_HRENTRADA := TO_CHAR(V_DTBATIDAPTO, 'HH24MI');
            
            V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRENTRADA, V_HRSAIDA);
            V_AUSENCIATOTAL := UTILIT.SOMA_HORA(V_AUSENCIATOTAL, V_HRTEMPOCALCULADO);

            OUT_RECORD.NRPONTO := NULL;
            OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
            OUT_RECORD.CDFUNC := P_CDFUNC;
            OUT_RECORD.IDOCORRENCIA := 'AUSENCIA';
            OUT_RECORD.DTHRENTRADAORIG := NULL;
            OUT_RECORD.DTHRENTRADA := V_DTHRENTRADA;
            OUT_RECORD.HRENTRADA := V_HRENTRADA;
            OUT_RECORD.DTHRSAIDAORIG := NULL;
            OUT_RECORD.DTHRSAIDA := V_DTHRSAIDA;
            OUT_RECORD.HRSAIDA := V_HRSAIDA;
            OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
            OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
            OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
            OUT_RECORD.HRTEMPOCALCULADO := V_HRTEMPOCALCULADO;
            OUT_RECORD.HRTEMPOCALCORIG := NULL;
            OUT_RECORD.IDALMOCO := 'N';
            OUT_RECORD.IDTEMPOABONADO := 'N';
            OUT_RECORD.IDTEMPONAOABON := 'S';
            OUT_RECORD.DSOBSERVACAO := NULL;
            OUT_RECORD.DSALTEENTRADA := NULL;
            OUT_RECORD.DSALTESAIDA := NULL;
            OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
            OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
            OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
            PIPE ROW (OUT_RECORD);
         END IF;

         IF (V_IDSENTBATIDA = '2') AND (V_IDULTSENTIDO <> '2') THEN
            V_DTHRSAIDA := V_DTBATIDAPTO;
            V_HRSAIDA := TO_CHAR(V_DTBATIDAPTO, 'HH24MI');
         END IF;

         V_IDULTSENTIDO := V_IDSENTBATIDA;
   END LOOP;
   CLOSE C_BATPONTO;
   
   OUT_RECORD.NRPONTO := V_NRPONTO;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'AUSENCIAALMOCO';
   OUT_RECORD.DTHRENTRADAORIG := NULL;
   OUT_RECORD.DTHRENTRADA := NULL;
   OUT_RECORD.HRENTRADA := NULL;
   OUT_RECORD.DTHRSAIDAORIG := NULL;
   OUT_RECORD.DTHRSAIDA := NULL;
   OUT_RECORD.HRSAIDA := NULL;
   OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
   OUT_RECORD.HRTEMPOCALCULADO := '0000';
   OUT_RECORD.HRTEMPOCALCORIG := NULL;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   PIPE ROW (OUT_RECORD);
   
   /* Tratamento para ESTAGIRIOS */ 
   IF V_CDTPADMISSAO IN ('B', 'E', 'O') THEN
      IF V_AUSENCIATOTAL > '0010' THEN
         V_AUSENCIAABONADA := '0010';
         V_AUSENCIANAOABON := UTILIT.SUBT_HORA2(V_AUSENCIATOTAL, '0010');
      ELSE
         V_AUSENCIAABONADA := V_AUSENCIATOTAL;
         V_AUSENCIANAOABON := '0000';
      END IF;
   ELSE
      V_AUSENCIAABONADA := '0000';
      V_AUSENCIANAOABON := V_AUSENCIATOTAL;
   END IF;
   
   OUT_RECORD.NRPONTO := V_NRPONTO;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'AUSENCIAABONADA';
   OUT_RECORD.DTHRENTRADAORIG := NULL;
   OUT_RECORD.DTHRENTRADA := NULL;
   OUT_RECORD.HRENTRADA := NULL;
   OUT_RECORD.DTHRSAIDAORIG := NULL;
   OUT_RECORD.DTHRSAIDA := NULL;
   OUT_RECORD.HRSAIDA := NULL;
   OUT_RECORD.HRAUSENCIAABONAORIG := V_AUSENCIAABONADA;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
   OUT_RECORD.HRTEMPOCALCULADO := V_AUSENCIAABONADA;
   OUT_RECORD.HRTEMPOCALCORIG := NULL;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   PIPE ROW (OUT_RECORD);
 
   OUT_RECORD.NRPONTO := NULL;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'AUSENCIANAOABONADA';
   OUT_RECORD.DTHRENTRADAORIG := NULL;
   OUT_RECORD.DTHRENTRADA := NULL;
   OUT_RECORD.HRENTRADA := NULL;
   OUT_RECORD.DTHRSAIDAORIG := NULL;
   OUT_RECORD.DTHRSAIDA := NULL;
   OUT_RECORD.HRSAIDA := NULL;
   OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := V_AUSENCIANAOABON;
   OUT_RECORD.HRTEMPOCALCULADO := V_AUSENCIANAOABON;
   OUT_RECORD.HRTEMPOCALCORIG := NULL;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   PIPE ROW (OUT_RECORD);
   
   OUT_RECORD.NRPONTO := NULL;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'AUSENCIATOTAL';
   OUT_RECORD.DTHRENTRADAORIG := NULL;
   OUT_RECORD.DTHRENTRADA := NULL;
   OUT_RECORD.HRENTRADA := NULL;
   OUT_RECORD.DTHRSAIDAORIG := NULL;
   OUT_RECORD.DTHRSAIDA := NULL;
   OUT_RECORD.HRSAIDA := NULL;
   OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
   OUT_RECORD.HRTEMPOCALCULADO := V_AUSENCIATOTAL;
   OUT_RECORD.HRTEMPOCALCORIG := V_AUSENCIATOTAL;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   PIPE ROW (OUT_RECORD);
   
   V_HORASREMUNERADAS := UTILIT.SUBT_HORA2(V_HORASREMUNERADAS, V_AUSENCIANAOABON);

   OUT_RECORD.NRPONTO := NULL;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'HORASREMUNERADAS';
   OUT_RECORD.DTHRENTRADAORIG := NULL;
   OUT_RECORD.DTHRENTRADA := NULL;
   OUT_RECORD.HRENTRADA := NULL;
   OUT_RECORD.DTHRSAIDAORIG := NULL;
   OUT_RECORD.DTHRSAIDA := NULL;
   OUT_RECORD.HRSAIDA := NULL;
   OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
   OUT_RECORD.HRTEMPOCALCULADO := V_HORASREMUNERADAS;
   OUT_RECORD.HRTEMPOCALCORIG := V_HORASREMUNERADAS;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   PIPE ROW (OUT_RECORD);

   IF V_HORASREMUNERADAS > V_QTHORASBASE THEN
      IF (V_CDTPADMISSAO IN ('S', 'P')) THEN
         V_DEBITOCREDITOHORAS := '0000';
      ELSE
         V_DEBITOCREDITOHORAS := UTILIT.SUBT_HORA2(V_HORASREMUNERADAS, V_QTHORASBASE);
      END IF;      
      
      /* #### Clculo atual considerando banco de horas #### */
      V_DEBITOCREDITOHORASEXTRA := ' 0000';
      
		/* #### Clculo considerando horas extras a partir de 02:00 horas positivas (funcionrios alocados em MG) 
		 * ou todo saldo positivo como hora extra (funcionrios alocados em outros estados) 
		 * ou todo saldo positivo como hora extra (Sb, Dom e Feriados - Solicitado por Gabrielle em 13/11/2014) #### */
      IF NOT (V_CDTPADMISSAO IN ('B', 'E', 'O')) THEN                
         IF V_SGESTADO_FUNC = 'MG' THEN
            IF (NOT (V_CDTPADMISSAO IN ('S', 'P'))) THEN 
					IF V_SAB_DOM_FERIADO = 'N' THEN 
						IF (V_DEBITOCREDITOHORAS > '0200') THEN
							V_DEBITOCREDITOHORASEXTRA := UTILIT.SUBT_HORA2(V_DEBITOCREDITOHORAS, '0200');
						END IF;
					ELSE
						V_DEBITOCREDITOHORASEXTRA := V_DEBITOCREDITOHORAS;
               END IF;            
            END IF;
         ELSE
            V_DEBITOCREDITOHORASEXTRA := V_DEBITOCREDITOHORAS;
         END IF;        
      END IF;   
   ELSE
      V_DEBITOCREDITOHORASEXTRA := ' 0000';
      IF V_HORASREMUNERADAS < V_QTHORASBASE THEN
         V_DEBITOCREDITOHORAS := '-'||UTILIT.SUBT_HORA2(V_QTHORASBASE, V_HORASREMUNERADAS);
      ELSE
         V_DEBITOCREDITOHORAS := ' '||UTILIT.SUBT_HORA2(V_QTHORASBASE, V_HORASREMUNERADAS);
      END IF;   
   END IF;   
   
   OUT_RECORD.NRPONTO := NULL;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'DEBITOCREDITOHORAS';
   OUT_RECORD.DTHRENTRADAORIG := NULL;
   OUT_RECORD.DTHRENTRADA := NULL;
   OUT_RECORD.HRENTRADA := NULL;
   OUT_RECORD.DTHRSAIDAORIG := NULL;
   OUT_RECORD.DTHRSAIDA := NULL;
   OUT_RECORD.HRSAIDA := NULL;
   OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
   OUT_RECORD.HRTEMPOCALCULADO := V_DEBITOCREDITOHORAS;
   OUT_RECORD.HRTEMPOCALCORIG := V_DEBITOCREDITOHORAS;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   PIPE ROW (OUT_RECORD);
   
   OUT_RECORD.NRPONTO := NULL;
   OUT_RECORD.CDEMPRESA := P_CDEMPRESA;
   OUT_RECORD.CDFUNC := P_CDFUNC;
   OUT_RECORD.IDOCORRENCIA := 'DEBCREDHORASEXTRA';
   OUT_RECORD.DTHRENTRADAORIG := NULL;
   OUT_RECORD.DTHRENTRADA := NULL;
   OUT_RECORD.HRENTRADA := NULL;
   OUT_RECORD.DTHRSAIDAORIG := NULL;
   OUT_RECORD.DTHRSAIDA := NULL;
   OUT_RECORD.HRSAIDA := NULL;
   OUT_RECORD.HRAUSENCIAABONAORIG := NULL;
   OUT_RECORD.HRAUSENCIAEXTABONORIG := NULL;
   OUT_RECORD.HRAUSENCIANAOABONORIG := NULL;
   OUT_RECORD.HRTEMPOCALCULADO := V_DEBITOCREDITOHORASEXTRA;
   OUT_RECORD.HRTEMPOCALCORIG := V_DEBITOCREDITOHORASEXTRA;
   OUT_RECORD.IDALMOCO := 'N';
   OUT_RECORD.IDTEMPOABONADO := 'N';
   OUT_RECORD.IDTEMPONAOABON := 'N';
   OUT_RECORD.DSOBSERVACAO := NULL;
   OUT_RECORD.DSALTEENTRADA := NULL;
   OUT_RECORD.DSALTESAIDA := NULL;
   OUT_RECORD.DSALTEAUSENCIAABONADA := NULL;
   OUT_RECORD.DSALTEAUSENCIAEXTABON := NULL;
   OUT_RECORD.DSALTEAUSENCIANAOABON := NULL;
   PIPE ROW (OUT_RECORD);
   
   RETURN;
END;
/

CREATE OR REPLACE 
FUNCTION FPA_ALMOCO_HORABASE
  (P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
   P_CDFUNC IN FUNCIONARIO.CDFUNC%TYPE,
   P_DTPONTO IN BATIDAPONTO.DTBATIDAPTO%TYPE,
   P_NRORG IN BATIDAPONTO.NRORG%TYPE DEFAULT 1,
   P_HORABASE IN CHAR DEFAULT '0') 
RETURN VARCHAR2

IS

   V_HBENTRADA VARCHAR2(4);
   V_HBSAIDAALMOCO VARCHAR2(4);
   V_HBVOLTAALMOCO VARCHAR2(4);
   V_HBSAIDA VARCHAR2(4);
   V_QTHORASBASE VARCHAR2(04);
   V_ALMOCOHORABASE VARCHAR2(4);
   V_RETORNO VARCHAR2(9);
   V_SAB_DOM_FERIADO VARCHAR2(01);
   
BEGIN
   V_ALMOCOHORABASE := '0000';
   
   /* Verificao de dia til/no til */
   BEGIN
      SELECT DECODE(FF.DTFERIFILI, NULL, 'N', 'S')
        INTO V_SAB_DOM_FERIADO
        FROM FERIFILI FF, FUNCIONARIO FU
       WHERE FU.NRORG = P_NRORG 
         AND FU.CDEMPRESA = P_CDEMPRESA 
         AND FU.CDFUNC = P_CDFUNC
         AND FF.NRORG = FU.NRORG
         AND FF.CDFILIAL = FU.CDFILIALOCFUNC
         AND FF.DTFERIFILI = P_DTPONTO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_SAB_DOM_FERIADO := 'N';
      END;
   END;

   BEGIN
      SELECT TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRENT1HBFUN, HBE.HRENT1HBEXC), 'HH24MI'), 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRSAI1HBFUN, HBE.HRSAI1HBEXC), 'HH24MI'), 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRENT2HBFUN, HBE.HRENT2HBEXC), 'HH24MI'),
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRSAI2HBFUN, HBE.HRSAI2HBEXC), 'HH24MI')
        INTO V_HBENTRADA, V_HBSAIDAALMOCO, V_HBVOLTAALMOCO, V_HBSAIDA
        FROM HORBASFU HBF, FUNCIONARIO FUN, FPA_HORABASEEXC HBE
       WHERE FUN.CDFUNC = P_CDFUNC
         AND FUN.CDEMPRESA = P_CDEMPRESA
         AND HBF.CDFUNC = FUN.CDFUNC
         AND HBF.CDEMPRESA = FUN.CDEMPRESA
         AND HBF.IDDIHBFUN = TO_CHAR(P_DTPONTO, 'D')
         AND HBF.DTINICHBFUN = (SELECT MAX(DTINICHBFUN)
                                  FROM HORBASFU
                                 WHERE CDFUNC = P_CDFUNC
                                   AND CDEMPRESA = P_CDEMPRESA
                                   AND DTINICHBFUN <= P_DTPONTO)
         AND HBE.CDTPADMISSAO(+) = FUN.CDTPADMISSAO
         AND HBE.DTHORABASEEXC(+) = P_DTPONTO;

      IF (V_HBENTRADA IS NULL) OR (V_SAB_DOM_FERIADO = 'S') THEN
         V_QTHORASBASE := '0000';
         V_ALMOCOHORABASE := '0000';
      ELSE
         IF V_HBVOLTAALMOCO IS NULL THEN
            V_QTHORASBASE := UTILIT.SUBT_HORA2(V_HBSAIDAALMOCO, V_HBENTRADA);
            V_ALMOCOHORABASE := '0000';
         ELSE
            V_QTHORASBASE := UTILIT.SOMA_HORA(UTILIT.SUBT_HORA2(V_HBSAIDAALMOCO, V_HBENTRADA), UTILIT.SUBT_HORA2(V_HBSAIDA, V_HBVOLTAALMOCO));
            V_ALMOCOHORABASE := UTILIT.SUBT_HORA2(V_HBVOLTAALMOCO, V_HBSAIDAALMOCO);
         END IF;   
      END IF;      
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_QTHORASBASE := '0000';
         V_ALMOCOHORABASE := '0000';
      END;
   END;
   
   V_RETORNO := V_ALMOCOHORABASE;
   IF (P_HORABASE <> '0') THEN
      V_RETORNO := V_RETORNO||'|'||V_QTHORASBASE;
   END IF;
   V_RETORNO := TRIM(V_RETORNO);
   
   RETURN (V_RETORNO);
END;
/
